/*
  labcomm2014_ioctl.h -- labcomm2014 ioctl declarations

  Copyright 2013 Anders Blomdell <anders.blomdell@control.lth.se>

  This file is part of LabComm.

  LabComm is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LabComm is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LABCOMM2014_IOCTL_H___
#define __LABCOMM2014_IOCTL_H___

#include "labcomm2014.h"

/*
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * | |   |                         |               |               |  
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  | |   |                         |               |
 *  | |   |                         |               +- number    (8)
 *  | |   |                         +----------------- type      (8)
 *  | |   +------------------------------------------- size      (13)
 *  | +----------------------------------------------- direction (2)
 *  +------------------------------------------------- signature (1)
 *  
 * type 0-31     are reserved for labcomm2014 library use
 */


#define LABCOMM2014_IOC_NRBITS      8
#define LABCOMM2014_IOC_TYPEBITS    8
#define LABCOMM2014_IOC_SIZEBITS   13
#define LABCOMM2014_IOC_DIRBITS     2
#define LABCOMM2014_IOC_SIGBITS     1
#define LABCOMM2014_IOC_NRMASK     ((1 << LABCOMM2014_IOC_NRBITS)-1)
#define LABCOMM2014_IOC_TYPEMASK   ((1 << LABCOMM2014_IOC_TYPEBITS)-1)
#define LABCOMM2014_IOC_SIZEMASK   ((1 << LABCOMM2014_IOC_SIZEBITS)-1)
#define LABCOMM2014_IOC_DIRMASK    ((1 << LABCOMM2014_IOC_DIRBITS)-1)
#define LABCOMM2014_IOC_SIGMASK    ((1 << LABCOMM2014_IOC_SIGBITS)-1)
#define LABCOMM2014_IOC_NRSHIFT    0
#define LABCOMM2014_IOC_TYPESHIFT  (LABCOMM2014_IOC_NRSHIFT+LABCOMM2014_IOC_NRBITS)
#define LABCOMM2014_IOC_SIZESHIFT  (LABCOMM2014_IOC_TYPESHIFT+LABCOMM2014_IOC_TYPEBITS)
#define LABCOMM2014_IOC_DIRSHIFT   (LABCOMM2014_IOC_SIZESHIFT+LABCOMM2014_IOC_SIZEBITS)
#define LABCOMM2014_IOC_SIGSHIFT   (LABCOMM2014_IOC_DIRSHIFT+LABCOMM2014_IOC_DIRBITS)

#define LABCOMM2014_IOC_NOSIG       0U
#define LABCOMM2014_IOC_USESIG      1U

#define LABCOMM2014_IOC_NONE        0U
#define LABCOMM2014_IOC_WRITE       1U
#define LABCOMM2014_IOC_READ        2U

#define LABCOMM2014_IOC(signature,dir,type,nr,size)	  \
  (((signature) << LABCOMM2014_IOC_SIGSHIFT) |	  \
   ((dir)       << LABCOMM2014_IOC_DIRSHIFT) |	  \
   ((size)      << LABCOMM2014_IOC_SIZESHIFT) |	  \
   ((type)      << LABCOMM2014_IOC_TYPESHIFT) |	  \
   ((nr)        << LABCOMM2014_IOC_NRSHIFT))	  

#define LABCOMM2014_IOC_SIG(nr) \
  (((nr) >> LABCOMM2014_IOC_SIGSHIFT) & LABCOMM2014_IOC_SIGMASK)
#define LABCOMM2014_IOC_DIR(nr) \
  (((nr) >> LABCOMM2014_IOC_DIRSHIFT) & LABCOMM2014_IOC_DIRMASK)
#define LABCOMM2014_IOC_SIZE(nr) \
  (((nr) >> LABCOMM2014_IOC_SIZESHIFT) & LABCOMM2014_IOC_SIZEMASK)
#define LABCOMM2014_IOC_TYPE(nr) \
  (((nr) >> LABCOMM2014_IOC_TYPESHIFT) & LABCOMM2014_IOC_TYPEMASK)
#define LABCOMM2014_IOC_NR(nr) \
  (((nr) >> LABCOMM2014_IOC_NRSHIFT) & LABCOMM2014_IOC_NRMASK)

#define LABCOMM2014_IO(type,nr)						\
  LABCOMM2014_IOC(LABCOMM2014_IOC_NOSIG,LABCOMM2014_IOC_NONE,type,nr,0)
#define LABCOMM2014_IOR(type,nr,size)					\
  LABCOMM2014_IOC(LABCOMM2014_IOC_NOSIG,LABCOMM2014_IOC_READ,type,nr,sizeof(size))
#define LABCOMM2014_IOW(type,nr,size)					\
  LABCOMM2014_IOC(LABCOMM2014_IOC_NOSIG,LABCOMM2014_IOC_WRITE,type,nr,sizeof(size))
#define LABCOMM2014_IOS(type,nr)                                    \
  LABCOMM2014_IOC(LABCOMM2014_IOC_USESIG,LABCOMM2014_IOC_READ,type,nr,0)
#define LABCOMM2014_IOSR(type,nr,size)					\
  LABCOMM2014_IOC(LABCOMM2014_IOC_USESIG,LABCOMM2014_IOC_READ,type,nr,sizeof(size))
#define LABCOMM2014_IOSW(type,nr,size)					\
  LABCOMM2014_IOC(LABCOMM2014_IOC_USESIG,LABCOMM2014_IOC_WRITE,type,nr,sizeof(size))

#define LABCOMM2014_IOCTL_WRITER_GET_BYTES_WRITTEN \
  LABCOMM2014_IOR(0,1,int)
#define LABCOMM2014_IOCTL_WRITER_GET_BYTE_POINTER \
  LABCOMM2014_IOR(0,2,void*)

#endif
